@@ -74,6 +74,10 @@ class RowSet(object):
         self.__changes = []
         self.__parent = parent
         self.__offset = offset
+        self.total_rows = total_rows
+        
+    def raw_rows(self):
+        return self.__rows
     
     def keys(self):
         return [x['key'] for x in self.__rows]
@@ -121,12 +125,14 @@ class RowSet(object):
     
     def __contains__(self, obj):
         if type(obj) is str:
-            return obj in (x['id'] for x in self.__rows) 
+            return obj in (x['id'] for x in self.__rows)
         else:
             return obj in (x['value'] for x in self.__rows)
     
     def __getitem__(self, i):
         if type(i) is int:
+            if i > len(self.__rows):
+                raise IndexError("out of range")
             if ( type(self.__rows[i]) is dict ) and (
                  type(self.__rows[i]) is not Document ) and ( 
                  type(self.__rows[i]['value']) is dict ) and ( 
@@ -139,6 +145,12 @@ class RowSet(object):
                         
         else:
             return RowSet(self.__db, [r for r in self.__rows if r['key'] == i], parent=self)
+    
+    def get(self, i, default=None):
+        try:
+            return self[i]
+        except IndexError:
+            return default
         
     def __setattr__(self, name, obj):
         if name.startswith("__") or "_RowSet__db":
@@ -164,6 +176,9 @@ class RowSet(object):
 #     def __len__(self):
 #         return len(self.result["rows"])
 
+
+class ViewException(Exception): pass
+
 class View(object):
     def __init__(self, db, path):
         self.db = db
@@ -174,8 +189,11 @@ class View(object):
         #         kwargs[k] = str(v).lower()
         #     if k in ['key', 'startkey', 'endkey']:
         #         kwargs[k] = json.dumps(v)
+        qs = {}
+        for k, v in kwargs.items():
+            if 'docid' not in k and k != 'stale': qs[k] = json.dumps(v)
+            else: qs[k] = v
         
-        qs = dict([(k,json.dumps(v),) for k,v in kwargs.items() if 'docid' not in k and k != 'stale'])
         query_string = urllib.urlencode(qs)
         if len(query_string) is not 0:
             path = self.path + '?' + query_string
@@ -187,11 +205,12 @@ class View(object):
         else:
             response = self.db.http.post(path, body=json.dumps({'keys':keys}))
         
-        assert response.status == 200
         result = json.loads(response.body)
-        return RowSet(self.db, result['rows'], offset=result.get('offset', None), 
-                       total_rows=result.get('total_rows'))
-        
+        if response.status == 200:
+            return RowSet(self.db, result['rows'], offset=result.get('offset', None), 
+                           total_rows=result.get('total_rows'))
+        else:
+            raise ViewException(result)
 
 class Design(object):
     def __init__(self, db, _id):
@@ -215,11 +234,11 @@ class Views(object):
         
     def temp_view(self, map, reduce=None, **kwargs):
         view = {"map":map}
-        if reduce:
+        if type(reduce) is str:
             view['reduce'] = reduce
         body = json.dumps(view)
         if len(kwargs) is 0:
-            uri = self.db.uri+'_temp_view'
+            path = self.db.uri+'_temp_view'
         else:
             for k, v in kwargs.items():
                 if type(v) is bool:
@@ -235,7 +254,7 @@ class Views(object):
             return RowSet(self.db, result['rows'], offset=result['offset'], 
                            total_rows=result['total_rows'])
         else:
-            raise TempViewException('Status: '+str(response.status)+'\nReason: '+response['reason']+'\nBody: '+response.body)
+            raise TempViewException('Status: '+str(response.status)+'\nBody: '+response.body)
     
     def all(self, keys=None, include_docs=True, **kwargs):
         kwargs['include_docs'] = include_docs
@@ -289,7 +308,10 @@ def deletedb(arg):
 
 class Database(object):
     def __init__(self, uri, http=None, http_engine=None, cache=None):
+        if not uri.endswith('/'):
+            uri += '/'
         self.uri = uri
+        
         if type(http) is httplib2.Http:
             self.http = Httplib2Client(uri, http_override=http)
         elif http_engine is None:
